==> redpesk-cli_3.15.0.orig.tar.gz: redpesk-cli-3.15.0/cmd/branches.go <==
/*
Copyright © 2022-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage
 Licensees holding valid commercial IoT.bzh licenses may use this file in
 accordance with the commercial license agreement provided with the
 Software or, alternatively, in accordance with the terms contained in
 a written agreement between you and The IoT.bzh Company. For licensing terms
 and conditions see https://www.iot.bzh/terms-conditions. For further
 information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage
 Alternatively, this file may be used under the terms of the GNU General
 Public license version 3. This license is as published by the Free Software
 Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
 of this file. Please review the following information to ensure the GNU
 General Public License requirements will be met
 https://www.gnu.org/licenses/gpl-3.0.html.
$RP_END_LICENSE$
*/

package cmd

import (
	"errors"
	"fmt"
	"strings"
	"text/tabwriter"

	"git.ovh.iot/redpesk/redpesk-infra/rp-webserver/actions/rpapiv1"
	"github.com/gofrs/uuid"
	"github.com/spf13/cobra"
)

// --------------------------------------------------------------
// Completion functions -----------------------------------------
// --------------------------------------------------------------

// getAppBranchesAsTab gets all branches that are available for
// a given application. This function works only if the user has passed
// (at least) the app ID. It is even better if a project ID is given
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :param toComplete: String passed by the user to complete
// :type toComplete: string
// :returns: []string, cobra.ShellCompDirective -- the tab result and the shell completion directive
func getAppBranchesAsTab(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
	branchesList := []string{}

	// Get the application UUID from what the user gave us
	appUUID, _, err := getAppUUIDAndSlug(cmd, args)
	if err != nil {
		return branchesList, cobra.ShellCompDirectiveNoFileComp
	}

	// Now we have an app ID we can retrieve the possible branches
	branchList := rpapiv1.RPBranchList{}
	if err := bckAPI.ApplicationGetBranchesList(appUUID.String(), &branchList); err != nil {
		return branchesList, cobra.ShellCompDirectiveNoFileComp
	}

	// We got correctly the branches list, fill the memberList tab
	for _, branch := range branchList.ListBranches {
		branchesList = append(branchesList, branch.Name)
	}

	return branchesList, cobra.ShellCompDirectiveNoFileComp
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Command functions --------------------------------------------
// --------------------------------------------------------------

// appCreateOrSwitchBranch allows to create a new branch in the given
// app or to switch to an existing branch
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went wrong, nil in the other case
func appCreateOrSwitchBranch(cmd *cobra.Command, args []string) error {

	// Retrieve the app UUID and its Slug
	appUUID, appSlug, err := getAppUUIDAndSlug(cmd, args)
	if err != nil {
		return err
	}

	// Get the value for the source branch
	srcBranch, err := cmd.Flags().GetString(appsBranchSourceFlag)
	if err != nil {
		logFlagError(appsBranchSourceFlag)
		return err
	}

	// Get the value for the destination branch
	dstBranch, err := cmd.Flags().GetString(appsBranchDestinationFlag)
	if err != nil {
		logFlagError(appsBranchDestinationFlag)
		return err
	}

	// Get the value for the default flag
	defaultBranch, err := cmd.Flags().GetBool(appsBranchDefaultFlag)
	if err != nil {
		logFlagError(appsBranchDefaultFlag)
		return err
	}

	// Get the value for the not-active flag
	notActive, err := cmd.Flags().GetBool(appsBranchNotActiveFlag)
	if err != nil {
		logFlagError(appsBranchNotActiveFlag)
		return err
	}

	// Get the value for the distributions flag
	distrosSlice, err := cmd.Flags().GetStringSlice(appsBranchDistrosFlag)
	if err != nil {
		logFlagError(appsBranchDistrosFlag)
		return err
	}

	// Get the value for the reset-distros flag
	resetDistro, err := cmd.Flags().GetBool(appsBranchResetDistrosFlag)
	if err != nil {
		logFlagError(appsBranchResetDistrosFlag)
		return err
	}

	// Check that the user gave at least a value for the destination branch
	if dstBranch == "" {
		return errors.New(dstBranchEmptyMsg)
	}

	// Create a variable allowing to know if the linked distributions have changed
	distrosChanged := false

	// Handle distributions according to the case we are in
	// If it is a branch creation => take directly distros given by user
	// If it is a branch switch => several cases need to be process in the if here below
	if srcBranch == "" && resetDistro {
		// The user wants to reset the distributions linked to the branch
		distrosSlice = []string{}
		distrosChanged = true
	} else if srcBranch == "" && len(distrosSlice) == 0 {
		// The user does not want to change anything on the distributions linked to the branch
		// Get the current distributions for the destination branch
		distrosSlice, err = getBranchDistrosList(appUUID, dstBranch)
		if err != nil {
			return err
		}
	} else if srcBranch == "" && len(distrosSlice) > 0 {
		distrosChanged = true
	}

	// Request the backend to create the branch or to switch to an existing one
	rpBranch := rpapiv1.RPBranch{
		Source:        srcBranch,
		Destination:   dstBranch,
		Distributions: distrosSlice,
		Default:       defaultBranch,
		Active:        !notActive,
	}
	if err := bckAPI.ApplicationCreateOrSwitchBranch(appUUID.String(), &rpBranch); err != nil {
		return err
	}

	// Tell the user that everything went well
	if srcBranch == "" && !notActive {
		RPPrinter.CustomPrintF(appsBranchCmdAnsLine0, dstBranch, appSlug)
		if distrosChanged {
			RPPrinter.CustomPrintLn(appsBranchCmdAnsLine3)
		}
	} else if !notActive {
		RPPrinter.CustomPrintF(appsBranchCmdAnsLine1, dstBranch, appSlug)
	} else if notActive {
		RPPrinter.CustomPrintF(appsBranchCmdAnsLine2, dstBranch, appSlug)
	}

	return nil
}

// appBranchesList gets all the branches available for an app
// and prints it.
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went wrong, nil in the other case
func appBranchesList(cmd *cobra.Command, args []string) error {

	// Retrieve the true application ID from UUID or its slug
	appUUID, err := genericGetAppUUID(cmd, args)
	if err != nil {
		return err
	}

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Request the branches list for the given application
	branches := rpapiv1.RPBranchList{}
	if err = bckAPI.ApplicationGetBranchesList(appUUID.String(), &branches); err != nil {
		return err
	}

	// Display the list to the user
	err = RPPrinter.CustomPrintObject(branches, verbose, displayBranches)

	return err
}

// appBranchesDelete delete one of the application branches
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went wrong, nil in the other case
func appBranchesDelete(cmd *cobra.Command, args []string) error {

	// Retrieve the app UUID and its Slug
	appUUID, appSlug, err := getAppUUIDAndSlug(cmd, args)
	if err != nil {
		return err
	}

	// Get the name of the branch to delete
	branchToDel, err := cmd.Flags().GetString(appsBranchFlag)
	if err != nil {
		logFlagError(appsBranchFlag)
		return err
	}

	// Check that the user gave a correct value for the branch to delete
	if branchToDel == "" {
		return errors.New(delBranchEmptyMsg)
	}

	// Construct the right structure and send the request to the backend
	rpBranchToDel := rpapiv1.RPBranchName{Name: branchToDel}
	if err := bckAPI.ApplicationDeleteBranch(appUUID.String(), &rpBranchToDel); err != nil {
		return err
	}

	// Tell the user that everything went well
	RPPrinter.CustomPrintF(appsBranchesDelCmdAnsLine0, branchToDel, appSlug)
	return nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Utilities functions ------------------------------------------
// --------------------------------------------------------------

// getBranchDistrosList retrieves the list of associated distributions for a given
// branch of an application
//
// :param appUUID: UUID of the application
// :type appUUID: uuid.UUID
// :param branch: Name of the branch
// :type branch: string
// :returns: []string, error -- a slice of distributions as string and an error if something went wrong
func getBranchDistrosList(appUUID uuid.UUID, branch string) ([]string, error) {
	var err error

	// Request the branches list for the given application
	branches := rpapiv1.RPBranchList{}
	if err = bckAPI.ApplicationGetBranchesList(appUUID.String(), &branches); err != nil {
		return []string{}, err
	}

	// Look for the branch in the list
	for _, current_branch := range branches.ListBranches {
		if current_branch.Name == branch {
			return current_branch.Distributions, nil
		}
	}

	// If we reached here, the branch has not been found
	return []string{}, fmt.Errorf(branchNotFoundInAppMsg, branch)
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Display functions --------------------------------------------
// --------------------------------------------------------------

// displayBranches is a private function allowing to print several
// branches in the console
//
// :param branches: pointer to the branches to print
// :type branches: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :param writer: Writer to used to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func displayBranches(branches interface{}, verbose bool, writer *tabwriter.Writer) error {
	branchesToPrint := branches.(rpapiv1.RPBranchList)

	// Iterate over the branches to print
	for _, branch := range branchesToPrint.ListBranches {
		// Initialize the variables we will use
		lineToPrint := "  "
		isActiveBranch := false

		// Check if the current branch is the active one
		if branch.Name == branchesToPrint.PackageActiveBranch {
			// Print in green if active one??? (Like in GIT)
			lineToPrint = "* "
			isActiveBranch = true
		}

		// Add the name of the branch
		lineToPrint += branch.Name

		// Add additional tag if verbose mode is one
		if verbose {
			verboseTxtTab := []string{}
			if isActiveBranch {
				verboseTxtTab = append(verboseTxtTab, "[ACTIVE]")
			}
			if branch.Default {
				verboseTxtTab = append(verboseTxtTab, "[DEFAULT]")
			}
			if branch.Protected {
				verboseTxtTab = append(verboseTxtTab, "[PROTECTED]")
			}
			lineToPrint += ("\t " + strings.Join(verboseTxtTab, ","))

			// Add the distributions linked if there are some
			distrosTxt := strings.Join(branch.Distributions, ",")
			lineToPrint += "\t " + "(" + distrosTxt + ")"
		}

		// Finally, print the line
		fmt.Fprintln(writer, lineToPrint)
	}

	return nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------

==> redpesk-cli_3.18.3.orig.tar.gz: redpesk-cli-3.18.3/cmd/branches.go <==
/*
Copyright © 2022-2023 IoT.bzh Company
Contact: https://www.iot.bzh/licensing

This file is part of the redpesk-cli project, the redpesk command line tool.

$RP_BEGIN_LICENSE$
Commercial License Usage
 Licensees holding valid commercial IoT.bzh licenses may use this file in
 accordance with the commercial license agreement provided with the
 Software or, alternatively, in accordance with the terms contained in
 a written agreement between you and The IoT.bzh Company. For licensing terms
 and conditions see https://www.iot.bzh/terms-conditions. For further
 information use the contact form at https://www.iot.bzh/contact.

GNU General Public License Usage
 Alternatively, this file may be used under the terms of the GNU General
 Public license version 3. This license is as published by the Free Software
 Foundation and appearing in the file LICENSE.GPLv3 included in the packaging
 of this file. Please review the following information to ensure the GNU
 General Public License requirements will be met
 https://www.gnu.org/licenses/gpl-3.0.html.
$RP_END_LICENSE$
*/

package cmd

import (
	"errors"
	"fmt"
	"strings"
	"text/tabwriter"

	"git.ovh.iot/redpesk/redpesk-infra/rp-webserver/actions/rpapiv1"
	"github.com/gofrs/uuid"
	"github.com/spf13/cobra"
)

// --------------------------------------------------------------
// Completion functions -----------------------------------------
// --------------------------------------------------------------

// getAppBranchesAsTab gets all branches that are available for
// a given application. This function works only if the user has passed
// (at least) the app ID. It is even better if a project ID is given
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :param toComplete: String passed by the user to complete
// :type toComplete: string
// :returns: []string, cobra.ShellCompDirective -- the tab result and the shell completion directive
func getAppBranchesAsTab(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
	branchesList := []string{}

	// Get the application given by the user
	app, err := genericGetAppFromArgs(cmd, args, false)
	if err != nil {
		return branchesList, cobra.ShellCompDirectiveNoFileComp
	}

	// Now we have an app ID we can retrieve the possible branches
	branchList := rpapiv1.RPBranchList{}
	if err := bckAPI.ApplicationGetBranchesList(app.ID.String(), &branchList); err != nil {
		return branchesList, cobra.ShellCompDirectiveNoFileComp
	}

	// We got correctly the branches list, fill the memberList tab
	for _, branch := range branchList.ListBranches {
		branchesList = append(branchesList, branch.Name)
	}

	return branchesList, cobra.ShellCompDirectiveNoFileComp
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Command functions --------------------------------------------
// --------------------------------------------------------------

// appCreateOrSwitchBranch allows to create a new branch in the given
// app or to switch to an existing branch
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went wrong, nil in the other case
func appCreateOrSwitchBranch(cmd *cobra.Command, args []string) error {

	// Get the application given by the user
	app, err := genericGetAppFromArgs(cmd, args, false)
	if err != nil {
		return err
	}

	// Get the value for the source branch
	srcBranch, err := cmd.Flags().GetString(appsBranchSourceFlag)
	if err != nil {
		logFlagError(appsBranchSourceFlag)
		return err
	}

	// Get the value for the destination branch
	dstBranch, err := cmd.Flags().GetString(appsBranchDestinationFlag)
	if err != nil {
		logFlagError(appsBranchDestinationFlag)
		return err
	}

	// Get the value for the default flag
	defaultBranch, err := cmd.Flags().GetBool(appsBranchDefaultFlag)
	if err != nil {
		logFlagError(appsBranchDefaultFlag)
		return err
	}

	// Get the value for the not-active flag
	notActive, err := cmd.Flags().GetBool(appsBranchNotActiveFlag)
	if err != nil {
		logFlagError(appsBranchNotActiveFlag)
		return err
	}

	// Get the value for the distributions flag
	distrosSlice, err := cmd.Flags().GetStringSlice(appsBranchDistrosFlag)
	if err != nil {
		logFlagError(appsBranchDistrosFlag)
		return err
	}

	// Get the value for the reset-distros flag
	resetDistro, err := cmd.Flags().GetBool(appsBranchResetDistrosFlag)
	if err != nil {
		logFlagError(appsBranchResetDistrosFlag)
		return err
	}

	// Check that the user gave at least a value for the destination branch
	if dstBranch == "" {
		return errors.New(dstBranchEmptyMsg)
	}

	// Create a variable allowing to know if the linked distributions have changed
	distrosChanged := false

	// Handle distributions according to the case we are in
	// If it is a branch creation => take directly distros given by user
	// If it is a branch switch => several cases need to be process in the if here below
	if srcBranch == "" && resetDistro {
		// The user wants to reset the distributions linked to the branch
		distrosSlice = []string{}
		distrosChanged = true
	} else if srcBranch == "" && len(distrosSlice) == 0 {
		// The user does not want to change anything on the distributions linked to the branch
		// Get the current distributions for the destination branch
		distrosSlice, err = getBranchDistrosList(app.ID, dstBranch)
		if err != nil {
			return err
		}
	} else if srcBranch == "" && len(distrosSlice) > 0 {
		distrosChanged = true
	}

	// Request the backend to create the branch or to switch to an existing one
	rpBranch := rpapiv1.RPBranch{
		Source:        srcBranch,
		Destination:   dstBranch,
		Distributions: distrosSlice,
		Default:       defaultBranch,
		Active:        !notActive,
	}
	if err := bckAPI.ApplicationCreateOrSwitchBranch(app.ID.String(), &rpBranch); err != nil {
		return err
	}

	// Tell the user that everything went well
	if srcBranch == "" && !notActive {
		RPPrinter.CustomPrintF(appsBranchCmdAnsLine0, dstBranch, app.Slug)
		if distrosChanged {
			RPPrinter.CustomPrintLn(appsBranchCmdAnsLine3)
		}
	} else if !notActive {
		RPPrinter.CustomPrintF(appsBranchCmdAnsLine1, dstBranch, app.Slug)
	} else if notActive {
		RPPrinter.CustomPrintF(appsBranchCmdAnsLine2, dstBranch, app.Slug)
	}

	return nil
}

// appBranchesList gets all the branches available for an app
// and prints it.
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went wrong, nil in the other case
func appBranchesList(cmd *cobra.Command, args []string) error {

	// Get the application given by the user
	app, err := genericGetAppFromArgs(cmd, args, false)
	if err != nil {
		return err
	}

	// Get the verbose flag
	verbose, err := cmd.Flags().GetBool(mainVerboseFlag)
	if err != nil {
		logFlagError(mainVerboseFlag)
		return err
	}

	// Request the branches list for the given application
	branches := rpapiv1.RPBranchList{}
	if err = bckAPI.ApplicationGetBranchesList(app.ID.String(), &branches); err != nil {
		return err
	}

	// Display the list to the user
	err = RPPrinter.CustomPrintObject(branches, verbose, displayBranches)

	return err
}

// appBranchesDelete delete one of the application branches
//
// :param cmd: Pointer to the cobra command received
// :type cmd: *cobra.Command
// :param args: Arguments passed by the user to the command
// :type args: []string
// :returns: error -- an error if something went wrong, nil in the other case
func appBranchesDelete(cmd *cobra.Command, args []string) error {

	// Get the application given by the user
	app, err := genericGetAppFromArgs(cmd, args, false)
	if err != nil {
		return err
	}

	// Get the name of the branch to delete
	branchToDel, err := cmd.Flags().GetString(appsBranchFlag)
	if err != nil {
		logFlagError(appsBranchFlag)
		return err
	}

	// Check that the user gave a correct value for the branch to delete
	if branchToDel == "" {
		return errors.New(delBranchEmptyMsg)
	}

	// Construct the right structure and send the request to the backend
	rpBranchToDel := rpapiv1.RPBranchName{Name: branchToDel}
	if err := bckAPI.ApplicationDeleteBranch(app.ID.String(), &rpBranchToDel); err != nil {
		return err
	}

	// Tell the user that everything went well
	RPPrinter.CustomPrintF(appsBranchesDelCmdAnsLine0, branchToDel, app.Slug)
	return nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Utilities functions ------------------------------------------
// --------------------------------------------------------------

// getBranchDistrosList retrieves the list of associated distributions for a given
// branch of an application
//
// :param appUUID: UUID of the application
// :type appUUID: uuid.UUID
// :param branch: Name of the branch
// :type branch: string
// :returns: []string, error -- a slice of distributions as string and an error if something went wrong
func getBranchDistrosList(appUUID uuid.UUID, branch string) ([]string, error) {
	var err error

	// Request the branches list for the given application
	branches := rpapiv1.RPBranchList{}
	if err = bckAPI.ApplicationGetBranchesList(appUUID.String(), &branches); err != nil {
		return []string{}, err
	}

	// Look for the branch in the list
	for _, current_branch := range branches.ListBranches {
		if current_branch.Name == branch {
			return current_branch.Distributions, nil
		}
	}

	// If we reached here, the branch has not been found
	return []string{}, fmt.Errorf(branchNotFoundInAppMsg, branch)
}

// getActiveBranch retrieves the branch list from the redpesk factory
// and returns the current active branch
//
// :param appUUID: UUID of the application
// :type appUUID: uuid.UUID
// :returns: string, error -- the current active branch and an error if something went wrong
func getActiveBranch(appUUID uuid.UUID) (string, error) {
	var err error

	// Request the branches list for the given application
	branches := rpapiv1.RPBranchList{}
	if err = bckAPI.ApplicationGetBranchesList(appUUID.String(), &branches); err != nil {
		return "", err
	}

	Log.Debugln("[BRANCHES] Current active branch:", branches.PackageActiveBranch)

	// Return the name of the active branch
	return branches.PackageActiveBranch, nil
}

// switchToExistingBranch switch the given application to the given gitpkg branch
// the function check if the branch exists before doing anything
//
// :param appUUID: UUID of the application
// :type appUUID: uuid.UUID
// :param dstBranch: destination branch
// :type dstBranch: string
// :returns: error -- an error if something went wrong
func switchToExistingBranch(appUUID uuid.UUID, dstBranch string) error {
	var err error

	// First, check that the destination branch exist

	// Request the branches list for the given application
	branches := rpapiv1.RPBranchList{}
	if err = bckAPI.ApplicationGetBranchesList(appUUID.String(), &branches); err != nil {
		return err
	}

	branchExists := false
	isBranchDefault := false
	for _, branch := range branches.ListBranches {
		if branch.Name == dstBranch {
			branchExists = true
			isBranchDefault = branch.Default
			break
		}
	}

	if !branchExists {
		return fmt.Errorf("the destination branch named '%s' does not exist in the factory", dstBranch)
	}

	// The branch exists, so let's switch to it

	// Get the current distributions for the destination branch
	distrosSlice, err := getBranchDistrosList(appUUID, dstBranch)
	if err != nil {
		return err
	}

	rpBranch := rpapiv1.RPBranch{
		Source:        "",
		Destination:   dstBranch,
		Distributions: distrosSlice,
		Default:       isBranchDefault,
		Active:        true,
	}
	Log.Debugln("[BRANCHES] Switching to branch:", dstBranch)
	return bckAPI.ApplicationCreateOrSwitchBranch(appUUID.String(), &rpBranch)
}

// --------------------------------------------------------------
// --------------------------------------------------------------

// --------------------------------------------------------------
// Display functions --------------------------------------------
// --------------------------------------------------------------

// displayBranches is a private function allowing to print several
// branches in the console
//
// :param branches: pointer to the branches to print
// :type branches: interface{}
// :param verbose: True if the user wants verbosity in the answer
// :type verbose: bool
// :param writer: Writer to used to print the information
// :type writer: *tabwriter.Writer
// :returns: error -- an error if something went bad
func displayBranches(branches interface{}, verbose bool, writer *tabwriter.Writer) error {
	branchesToPrint := branches.(rpapiv1.RPBranchList)

	// Iterate over the branches to print
	for _, branch := range branchesToPrint.ListBranches {
		// Initialize the variables we will use
		lineToPrint := "  "
		isActiveBranch := false

		// Check if the current branch is the active one
		if branch.Name == branchesToPrint.PackageActiveBranch {
			// Print in green if active one??? (Like in GIT)
			lineToPrint = "* "
			isActiveBranch = true
		}

		// Add the name of the branch
		lineToPrint += branch.Name

		// Add additional tag if verbose mode is one
		if verbose {
			verboseTxtTab := []string{}
			if isActiveBranch {
				verboseTxtTab = append(verboseTxtTab, "[ACTIVE]")
			}
			if branch.Default {
				verboseTxtTab = append(verboseTxtTab, "[DEFAULT]")
			}
			if branch.Protected {
				verboseTxtTab = append(verboseTxtTab, "[PROTECTED]")
			}
			lineToPrint += ("\t " + strings.Join(verboseTxtTab, ","))

			// Add the distributions linked if there are some
			distrosTxt := strings.Join(branch.Distributions, ",")
			lineToPrint += "\t " + "(" + distrosTxt + ")"
		}

		// Finally, print the line
		fmt.Fprintln(writer, lineToPrint)
	}

	return nil
}

// --------------------------------------------------------------
// --------------------------------------------------------------
